#!/var/folders/sy/f16zz6x50xz3113nwtb9bvq00000gp/T/abs_60x7mdl7mm/croot/anaconda-navigator_1694074170916/_h_env_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_place/Anaconda-Navigator.app/Contents/MacOS/python
from subprocess import call

call(["/var/folders/sy/f16zz6x50xz3113nwtb9bvq00000gp/T/abs_60x7mdl7mm/croot/anaconda-navigator_1694074170916/_h_env_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_place/bin/anaconda-navigator"])
